package com.gic.cloud.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.cloud.dto.UserDTO;
import com.gic.cloud.entity.TabSysAccountGroup;
import com.gic.cloud.entity.TabSysUser;
import com.gic.cloud.qo.UserQO;
import com.gic.cloud.service.AccountGroupService;
import com.gic.cloud.service.UserApiService;
import com.gic.cloud.service.UserService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.constants.Constants;
import com.gic.enterprise.error.ErrorCode;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Service("userApiService")
public class UserApiServiceImpl implements UserApiService{
    @Autowired
    private UserService userService;
    @Autowired
    private AccountGroupService accountGroupService;

    @Override
    public ServiceResponse<Integer> saveUser(UserDTO dto) {
        dto.setUserId(null);
        ServiceResponse validParamResult = validParam(dto);
        if (!validParamResult.isSuccess()) {
            return ServiceResponse.failure(validParamResult.getCode(), validParamResult.getMessage());
        }
        return ServiceResponse.success(userService.saveUser(dto));
    }

    @Override
    public ServiceResponse<Void> editUser(UserDTO dto) {
        ServiceResponse validParamResult = validParam(dto);
        if (!validParamResult.isSuccess()) {
            return ServiceResponse.failure(validParamResult.getCode(), validParamResult.getMessage());
        }
        userService.editUser(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> deleteUser(Integer userId) {
        userService.deleteUser(userId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> deleteUserByAccountGroupId(Integer accountGroupId) {
        userService.deleteUserByAccountGroupId(accountGroupId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<UserDTO>> pageUser(UserQO userQO) {
        com.github.pagehelper.Page page = userService.pageUser(userQO);
        Page<UserDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, UserDTO.class);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<List<UserDTO>> listUser(UserQO userQO) {
        return ServiceResponse.success(Optional.ofNullable(EntityUtil.changeEntityListNew(UserDTO.class,
                userService.listUser(userQO))).orElse(Collections.EMPTY_LIST));
    }

    @Override
    public ServiceResponse<Void> bulkTransferAccountGroup(List<Integer> userIdList, Integer targetAccountGroupId) {
        TabSysAccountGroup accountGroup = accountGroupService.getAccountGroupById(targetAccountGroupId);
        if (accountGroup == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "账号分组不存在");
        }
        userService.bulkTransferAccountGroup(userIdList, targetAccountGroupId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<UserDTO>> listUserByPhone(String nationCode, String phone) {
        List<TabSysUser> list = userService.listUserByPhone(nationCode, phone);
        if (CollectionUtils.isNotEmpty(list)) {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(UserDTO.class, list));
        }
        return ServiceResponse.success(Collections.emptyList());
    }

    @Override
    public ServiceResponse<UserDTO> getUser(Integer userId) {
        return ServiceResponse.success(EntityUtil.changeEntityNew(UserDTO.class, userService.getUser(userId)));
    }

    private ServiceResponse<Void> validParam(UserDTO dto) {
        if (dto.getUserId() != null) {
            TabSysUser record = userService.getUser(dto.getUserId());
            if (record == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户ID不能为空");
            }
            dto.setEnterpriseId(record.getEnterpriseId());
        }
        Integer enterpriseId = dto.getEnterpriseId();
        if (enterpriseId == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID不能为空");
        }
        if (StringUtils.isBlank(dto.getUserName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户名称不能为空");
        }
        if (StringUtils.isBlank(dto.getNationCode())) {
            dto.setNationCode(Constants.NATION_CODE);
        }
        if (dto.getNationCode().equals(Constants.NATION_CODE) && !isPhone(dto.getPhone())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "手机号码非法");
        }
        if (!dto.getNationCode().equals(Constants.NATION_CODE) && StringUtils.isBlank(dto.getPhone())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "手机号码不能为空");
        }
        if (userService.isRepeatPhone(enterpriseId, dto.getUserId(), dto.getNationCode(), dto.getPhone())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "手机号码重复");
        }
        if (dto.getAccountGroupId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "账号分组不能为空");
        }
        if (dto.getDataAuthId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "数据权限集不能为空");
        }
        if (dto.getFunctionAuthId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "功能权限集不能为空");
        }
        return ServiceResponse.success();
    }

    private static boolean isPhone(String phone) {
        String regex = "^((13[0-9])|(14[0|5|6|7|9])|(15[0-3])|(15[5-9])|(16[6|7])|(17[2|3|5|6|7|8])|(18[0-9])|(19[1|8|9]))\\d{8}$";
        if (StringUtils.isBlank(phone) || phone.length() != 11) {
            return false;
        } else {
            Pattern p = Pattern.compile(regex);
            Matcher m = p.matcher(phone);
            boolean isMatch = m.matches();
            if (isMatch) {
                return true;
            } else {
                return false;
            }
        }
    }
}
